package com.diagbot.web;


import com.diagbot.annotation.SysLogger;
import com.diagbot.client.bean.GdbResponse;
import com.diagbot.client.bean.ResponseData;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.ClinicalFacade;
import com.diagbot.vo.SearchVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.Map;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author Mark Huang
 * @since 2019-03-04
 */
@RestController
@RequestMapping("/clinicaldata")
@Api(value = "临床数据处理API", tags = { "临床数据处理API" })

public class ClinicalController {
    @Autowired
    private ClinicalFacade clinicalFacade;

    @ApiOperation(value = "临床数据处理", notes = "")
    @PostMapping("/processData")
    @SysLogger("processData")
    public RespDTO<ResponseData> processData(@Valid @RequestBody SearchVo searchVo) {

        ResponseData data = clinicalFacade.processClinicalData(searchVo);

        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "获取量表", notes = "")
    @PostMapping("/scale")
    @SysLogger("scale")
    public RespDTO<Map<String, Object>> scale(@RequestBody SearchVo searchVo) {
        Map<String, Object> data = clinicalFacade.scale(searchVo);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "高危诊断", notes = "")
    @PostMapping("/highRisk")
    @SysLogger("highRisk")
    public RespDTO<GdbResponse> highRisk(@RequestBody SearchVo searchVo) {
        GdbResponse data = clinicalFacade.highRiskPageData(searchVo);
        return RespDTO.onSuc(data);
    }
}
