package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.DisDTO;
import com.diagbot.dto.DisTypeDTO;
import com.diagbot.entity.DisType;
import com.diagbot.entity.QuestionInfo;
import com.diagbot.enums.DisTypeEnum;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.QuestionTypeEnum;
import com.diagbot.service.impl.DisTypeServiceImpl;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

/**
 * @Description: 诊断类型
 * @author: zhoutg
 * @time: 2018/11/19 13:19
 */
@Component
public class DisTypeFacade extends DisTypeServiceImpl {
    @Autowired
    private QuestionFacade questionFacade;

    /**
     * 返回诊断类型
     *
     * @return
     */
    public DisTypeDTO getDisTypeFac() {
        DisTypeDTO res = new DisTypeDTO();
        List<DisDTO> data = this.getDisType();
        Map<String, List<DisDTO>> map = EntityUtil.makeEntityListMap(data, "type");
        if (map != null) {
            List<DisDTO> chr = map.get(DisTypeEnum.CHRONIC.getKey()); //放入慢病列表
            if (ListUtil.isNotEmpty(chr)) {
                res.setChronic(chr);
            }
            List<DisDTO> eme = map.get(DisTypeEnum.EMERGENCY.getKey()); //放入急诊列表
            if (ListUtil.isNotEmpty(eme)) {
                res.setEmergency(eme);
            }
        }
        return res;
    }

    /**
     * 根据诊断id判断类型
     *
     * @param diseaseId
     * @param type
     * @return
     */
    public Boolean judgeByDisId(Long diseaseId, Integer type) {
        QueryWrapper<DisType> disTypeQueryWrapper = new QueryWrapper<>();
        disTypeQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("dis_id", diseaseId)
                .eq("type", type);
        DisType disType = this.getOne(disTypeQueryWrapper);
        if (disType == null) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * 根据诊断名称判断类型
     *
     * @param disName
     * @param type
     * @return
     */
    public Boolean judgeByDisName(String disName, Integer type) {
        QueryWrapper<QuestionInfo> questionInfoQueryWrapper = new QueryWrapper<>();
        questionInfoQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("tag_name", disName)
                .eq("type", QuestionTypeEnum.Disease.getKey());
        QuestionInfo disease = questionFacade.getOne(questionInfoQueryWrapper);
        if (disease == null) {
            return false;
        } else {
            return judgeByDisId(disease.getId(), type);
        }
    }
}
