package com.diagbot.facade;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.client.AiptServiceClient;
import com.diagbot.client.TranServiceClient;
import com.diagbot.client.bean.FeatureRate;
import com.diagbot.client.bean.GdbResponse;
import com.diagbot.client.bean.MedicalIndication;
import com.diagbot.client.bean.Response;
import com.diagbot.client.bean.ResponseData;
import com.diagbot.client.bean.SearchData;
import com.diagbot.dto.EMRQuestionDTO;
import com.diagbot.dto.EvaluationModuleMappingDTO;
import com.diagbot.dto.MedicalIndicationDTO;
import com.diagbot.dto.PushDTO;
import com.diagbot.dto.PushEMRDTO;
import com.diagbot.dto.PushKYJDTO;
import com.diagbot.dto.QuestionDTO;
import com.diagbot.dto.ScaleDTO;
import com.diagbot.entity.DeptInfo;
import com.diagbot.entity.DeptVital;
import com.diagbot.entity.QuestionInfo;
import com.diagbot.entity.ScaleContent;
import com.diagbot.enums.DisTypeEnum;
import com.diagbot.enums.FeatureTypeEnum;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.LisSourceEnum;
import com.diagbot.enums.QuantitativeTypeEnum;
import com.diagbot.enums.QuestionTypeEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.util.ArrayUtil;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.FastJsonUtils;
import com.diagbot.util.ListUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.EMRPushVO;
import com.diagbot.vo.HosCodeVO;
import com.diagbot.vo.LisKYJVO;
import com.diagbot.vo.LisResult;
import com.diagbot.vo.LisResultVO;
import com.diagbot.vo.PushKYJVO;
import com.diagbot.vo.PushVO;
import com.diagbot.vo.QuestionVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2018/11/20 11:18
 */
@Component
public class PushFacade {
    @Autowired
    private QuestionFacade questionFacade;
    @Autowired
    private DeptInfoFacade deptInfoFacade;
    @Autowired
    private DeptVitalFacade deptVitalFacade;
    @Autowired
    private TreatmentFacade treatmentFacade;
    @Autowired
    private LisMappingFacade lisMappingFacade;
    @Autowired
    private DisTypeFacade disTypeFacade;
    @Autowired
    private ScaleContentFacade scaleContentFacade;
    @Autowired
    private EvaluationModuleMappingFacade evaluationModuleMappingFacade;
    @Autowired
    private TranServiceClient tranServiceClient;
    @Autowired
    private AiptServiceClient aiptServiceClient;

    /**
     * 拼装数据
     *
     * @param pushVO
     * @return SearchData
     */
    public SearchData assembleData(PushVO pushVO) {
        SearchData searchData = new SearchData();
        searchData.setAge(pushVO.getAge());
        searchData.setSymptom(pushVO.getSymptom());
        searchData.setVital(pushVO.getVital());
        searchData.setPacs(pushVO.getPacs());
        searchData.setPast(pushVO.getPast());
        searchData.setOther(pushVO.getOther());
        searchData.setDiag(pushVO.getDiag());
        searchData.setDisType(pushVO.getDisType());
        searchData.setAdverseEvent(pushVO.getAdverseReactions());
        searchData.setScaleName(pushVO.getScaleName());
        searchData.setIndications(pushVO.getMedicalIndications());
        searchData.setFeatureType(pushVO.getFeatureType());
        searchData.setSysCode("1");
        searchData.setLength(10);

        switch (pushVO.getSex()) {
            case 1:
                searchData.setSex("M");
                break;
            case 2:
                searchData.setSex("F");
                break;
            default:
                searchData.setSex("A");
                break;
        }

        //化验项映射,转公表
        if (ListUtil.isNotEmpty(pushVO.getLis())) {
            Map<String, Map<String, String>> lisConfigMap = lisMappingFacade.getLisMapping_NotEmptyItemName();
            Map<String, String> lisConfigWithEmptyItemNameMap = lisMappingFacade.getLisConfig_EmptyItemName();
            for (LisResultVO lisResultVO : pushVO.getLis()) {
                //source=0 icss页面数据，source=1 外部导入数据，外部导入数据不处理
                if (!lisResultVO.getSource().equals(LisSourceEnum.Inner.getKey())) {
                    continue;
                }
                if (StringUtil.isNotBlank(lisResultVO.getName())) {
                    if (StringUtil.isNotBlank(lisResultVO.getDetailName())) {
                        Map<String, String> detailMap = lisConfigMap.get(lisResultVO.getName());
                        if (null != detailMap && detailMap.size() > 0 && StringUtil.isNotBlank(detailMap.get(lisResultVO.getDetailName()))) {
                            lisResultVO.setUniqueName(detailMap.get(lisResultVO.getDetailName()));
                        }
                    } else {
                        lisResultVO.setUniqueName(lisConfigWithEmptyItemNameMap.get(lisResultVO.getName()));
                    }
                }
            }
        }
        List<LisResult> lisResultList = BeanUtil.listCopyTo(pushVO.getLis(), LisResult.class);
        searchData.setLisArr(lisResultList);

        return searchData;
    }

    /**
     * 中间层接口
     *
     * @param searchData
     * @return ResponseData
     */
    public ResponseData pushAipt(SearchData searchData) {
        Response<ResponseData> res = aiptServiceClient.aiptData(searchData);
        if (null == res || null == res.getData()) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "中间层没有返回结果");
        }
        return res.getData();
    }

    /**
     * 获取量表
     *
     * @param searchData
     * @return
     */
    public Map<String, Object> scale(SearchData searchData) {
        Response<Map<String, Object>> res = aiptServiceClient.scale(searchData);
        if (null == res || null == res.getData()) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "中间层没有返回结果");
        }
        return res.getData();
    }

    /**
     * 推理接口
     *
     * @param pushVO
     * @return
     */
    public PushDTO pushInner(PushVO pushVO, Integer mode) {
        PushDTO pushDTO = new PushDTO();
        SearchData aiSearchData = assembleData(pushVO);
        ResponseData data = pushAipt(aiSearchData);

        //大数据返回内容
        List<FeatureRate> dis = data.getDis();

        String featureType = pushVO.getFeatureType();
        String[] featureTypes = featureType.split(",|，");
        Set<String> featureTypeSet = new HashSet(Arrays.asList(featureTypes));

        //症状 标签列表
        if (featureTypeSet.contains(String.valueOf(FeatureTypeEnum.Symptom.getKey()))) {
            List<FeatureRate> symptom = data.getSymptom();
            if (ListUtil.isNotEmpty(symptom)) {
                List<String> nameList = symptom.stream().map(featureRate -> featureRate.getFeatureName()).collect(Collectors.toList());
                List<QuestionDTO> symptomDTO = getTagList(nameList, pushVO, FeatureTypeEnum.Symptom.getKey());
                pushDTO.setSymptom(symptomDTO);
            }
        }
        //其他史 标签列表
        if (featureTypeSet.contains(String.valueOf(FeatureTypeEnum.Other.getKey()))) {
            List<FeatureRate> other = data.getHistory();
            if (ListUtil.isNotEmpty(other)) {
                List<String> nameList = other.stream().map(featureRate -> featureRate.getFeatureName()).collect(Collectors.toList());
                List<QuestionDTO> otherDTO = getTagList(nameList, pushVO, FeatureTypeEnum.Other.getKey());
                pushDTO.setOther(otherDTO);
            }
        }
        //查体 返回模板（标签列表）+标记标签
        if (featureTypeSet.contains(String.valueOf(FeatureTypeEnum.Vital.getKey()))) {
            List<FeatureRate> vitals = data.getVitals();
            //没有推送信息时，默认取全科模板
            String deptName = "";
            if (dis != null && dis.size() > 0) {
                for (FeatureRate featureRate : dis) {
                    if (StringUtil.isNotBlank(featureRate.getExtraProperty())) {
                        deptName = featureRate.getExtraProperty();
                        break;
                    }
                }
                //deptName = dis.get(0).getExtraProperty();
            }
            if (StringUtil.isBlank(deptName)) {
                deptName = "全科";
            }
            List<QuestionDTO> vitalDTO = getVitalModule(deptName, pushVO);
            pushDTO.setVital(vitalDTO);

            //查体标记项
            if (ListUtil.isNotEmpty(vitals)) {
                List<String> nameList = vitals.stream().map(featureRate -> featureRate.getFeatureName()).collect(Collectors.toList());
                List<QuestionDTO> importantVitalDTO = getTagList(nameList, pushVO, FeatureTypeEnum.Vital.getKey());
                List<Long> markedVitalIds = Lists.newLinkedList();
                for (QuestionDTO vital : importantVitalDTO) {
                    if (vital.getId() != null && (!markedVitalIds.contains(vital.getId()))) {
                        markedVitalIds.add(vital.getId());
                    }
                }
                pushDTO.setMarkedVitalIds(markedVitalIds);
            }
        }
        //化验  标签列表
        if (featureTypeSet.contains(String.valueOf(FeatureTypeEnum.Lis.getKey()))) {
            List<FeatureRate> labs = data.getLabs();
            if (ListUtil.isNotEmpty(labs)) {
                //公表项转套餐项
                Map<String, List<String>> lisMapping = lisMappingFacade.getLisMappingByUniqueName();
                List<String> nameList = Lists.newLinkedList();
                for (FeatureRate featureRate : labs) {
                    if (ListUtil.isNotEmpty(lisMapping.get(featureRate.getFeatureName()))) {
                        //匹配出多个套餐项默认取第一个
                        String name = lisMapping.get(featureRate.getFeatureName()).get(0);
                        if (nameList.contains(name)) {
                            continue;
                        }
                        nameList.add(name);
                    }
                }
                List<QuestionDTO> labDTO = getTagList(nameList, pushVO, FeatureTypeEnum.Lis.getKey());
                pushDTO.setLab(labDTO);
            }
        }
        //辅检  标签列表
        if (featureTypeSet.contains(String.valueOf(FeatureTypeEnum.Pacs.getKey()))) {
            List<FeatureRate> pacs = data.getPacs();
            if (ListUtil.isNotEmpty(pacs)) {
                List<String> nameList = pacs.stream().map(featureRate -> featureRate.getFeatureName()).collect(Collectors.toList());
                List<QuestionDTO> pacsDTO = getTagList(nameList, pushVO, FeatureTypeEnum.Pacs.getKey());
                pushDTO.setPacs(pacsDTO);
            }
        }
        //诊断 返回分类+标签列表
        if (featureTypeSet.contains(String.valueOf(FeatureTypeEnum.Disease.getKey()))) {
            if (ListUtil.isNotEmpty(dis)) {
                List<String> nameList = dis.stream().map(featureRate -> featureRate.getFeatureName()).collect(Collectors.toList());
                Map<String, List<QuestionDTO>> disMapDTO = new LinkedHashMap<>();
                Map<String, List<FeatureRate>> disFeatureMap = new LinkedHashMap<>();
                //警惕
                Map<String, String> highRiskMap = new HashMap<>();
                highRiskMap.put("disease", String.join(",", nameList));
                SearchData searchData = new SearchData();
                searchData.setDiag(String.join(",", nameList));
                Response<GdbResponse> graphRes = aiptServiceClient.highRisk(searchData);
                if (graphRes != null) {
                    Map<String, String> graphResult = graphRes.getData().getResult();
                    if (graphResult.size() > 0) {
                        List<String> hrNameList = Lists.newLinkedList();
                        for (Map.Entry<String, String> entry : graphResult.entrySet()) {
                            if (entry.getValue().equals("1")) {
                                hrNameList.add(entry.getKey());
                            }
                        }
                        List<QuestionDTO> disDTO = getTagList(hrNameList, pushVO, FeatureTypeEnum.Disease.getKey());
                        disMapDTO.put("警惕", disDTO);
                    }
                }

                //诊断分类
                for (FeatureRate featureRate : dis) {
                    if (StringUtil.isBlank(featureRate.getDesc())) {
                        featureRate.setDesc("{\"可能诊断\":\"\"}");
                    }
                    Map<String, Object> descMap = FastJsonUtils.getJsonToMap(featureRate.getDesc());
                    for (String disClass : descMap.keySet()) {
                        List<FeatureRate> featureRateList = Lists.newLinkedList();
                        if (disFeatureMap.get(disClass) != null) {
                            featureRateList = disFeatureMap.get(disClass);
                        }
                        featureRateList.add(featureRate);
                        disFeatureMap.put(disClass, featureRateList);
                    }
                }
                for (Map.Entry<String, List<FeatureRate>> entry : disFeatureMap.entrySet()) {
                    List<String> nameListByDisClass = entry.getValue().stream().map(featureRate -> featureRate.getFeatureName()).collect(Collectors.toList());
                    List<QuestionDTO> disDTO = getTagList(nameListByDisClass, pushVO, FeatureTypeEnum.Disease.getKey());
                    disMapDTO.put(entry.getKey(), disDTO);
                }
                pushDTO.setDis(disMapDTO);
            }
        }
        //治疗方案
        if (featureTypeSet.contains(String.valueOf(FeatureTypeEnum.Treat.getKey()))) {
            Map<String, JSONObject> treat = data.getTreat();
            if (pushVO.getDiseaseId() == null) {
                throw new CommonException(CommonErrorCode.PARAM_ERROR, "请输入诊断id");
            }
            Map<String, Object> treatmentMap = treatmentFacade.getTreatment(treat, pushVO.getDiseaseId(), pushVO.getPatientId());
            pushDTO.setTreat(treatmentMap);
        }

        //量表内容 featuretype=21
        if (featureTypeSet.contains(String.valueOf(FeatureTypeEnum.Scale.getKey()))) {
            Map<String, Object> scale = scale(aiSearchData);
            String scaleContent = "";
            if (scale != null && scale.size() > 0) {
                scaleContent = JSON.toJSONString(scale);
            }
            List<ScaleContent> scaleContentList = scaleContentFacade.getContent(pushVO.getScaleId(), scaleContent);
            pushDTO.setScale(scaleContentList);
        }

        //管理评估 featuretype=11 && disType=1(慢病)
        if (featureTypeSet.contains(String.valueOf(FeatureTypeEnum.ManagementEvaluation.getKey()))) {
            Map<String, JSONObject> managementEvaluation = data.getManagementEvaluation();
            if (pushVO.getDisType().equals(DisTypeEnum.CHRONIC.getKey())) {
                /***********************测试数据********************************/
                if (managementEvaluation == null) {
                    managementEvaluation = new LinkedHashMap<>();
                    managementEvaluation.put("疗效评估", JSONObject.parseObject("{\"list\":[{\"className\":\"血压控制情况\",\"controltype\":1,\"details\":[{\"name\":\"正常\",\"isSelected\":1},{\"name\":\"1级\",\"isSelected\":0},{\"name\":\"2级\",\"isSelected\":0},{\"name\":\"3级\",\"isSelected\":0}]},{\"className\":\"BMI控制情况\",\"controltype\":2,\"details\":[{\"name\":\"体重过低\",\"isSelected\":1},{\"name\":\"体重达标\",\"isSelected\":0},{\"name\":\"肥胖前期\",\"isSelected\":0},{\"name\":\"肥胖\",\"isSelected\":0}]},{\"className\":\"血脂控制情况\",\"controltype\":2,\"details\":[{\"name\":\"高血脂\",\"isSelected\":1}]}]}"));
                    managementEvaluation.put("糖尿病分级管理", JSONObject.parseObject("{\"className\":\"血压控制情况\",\"controltype\":1,\"details\":[{\"name\":\"常规管理\",\"isSelected\":1},{\"name\":\"强化管理\",\"isSelected\":0}]}"));
                }
                /***********************正式流程********************************/
                List<EvaluationModuleMappingDTO> evaluationModule = evaluationModuleMappingFacade.getEvaluationModule(pushVO.getDiseaseId(), managementEvaluation);
                pushDTO.setEvaluationModule(evaluationModule);
            }
        }

        //推送指标 featuretype=22
        if (featureTypeSet.contains(String.valueOf(FeatureTypeEnum.MedicalIndication.getKey()))) {
            List<MedicalIndication> medicalIndicationList = data.getMedicalIndications();
            /***********************测试数据********************************/
            if (ListUtil.isEmpty(medicalIndicationList)) {
                String indicationStr = "[" +
                        "{\"name\":\"糖尿病周围神经病变\",\"type\":1,\"content\":{},\"scaleName\":\"密西根糖尿病周围神经病筛查表\"},{\"name\":\"高血脂\",\"type\":4,\"content\":{},\"scaleName\":\"\"}," +
                        "{\"name\":\"高血压\",\"type\":1,\"content\":{},\"scaleName\":\"高血压量表1\"}," +
                        "{\"name\":\"肾功能不全\",\"type\":2,\"content\":{},\"scaleName\":\"\"}]";
                String contentStr = "{\"type\":\"checkbox\",\"details\":[{\"name\":\"轻度\",\"isSelected\":0}," +
                        "{\"name\":\"中度\",\"isSelected\":1},{\"name\":\"重度\",\"isSelected\":0}]}";
                JSONObject contentJson = JSONObject.parseObject(contentStr);
                medicalIndicationList = FastJsonUtils.getJsonToList(indicationStr, MedicalIndication.class);
                medicalIndicationList.get(3).setContent(contentJson);
            }

            /***********************正式流程********************************/
            if (ListUtil.isNotEmpty(medicalIndicationList)) {
                List<MedicalIndicationDTO> medicalIndicationDTOList = BeanUtil.listCopyTo(medicalIndicationList, MedicalIndicationDTO.class);
                for (MedicalIndicationDTO medicalIndicationDTO : medicalIndicationDTOList) {
                    medicalIndicationDTO.setTagName(medicalIndicationDTO.getName());
                    //返回标签过滤年龄性别
                    QueryWrapper<QuestionInfo> questionInfoQueryWrapper = new QueryWrapper<>();
                    questionInfoQueryWrapper.le("age_begin", pushVO.getAge())
                            .ge("age_end", pushVO.getAge())
                            .and(i -> i.eq("sex_type", 3).or().eq("sex_type", pushVO.getSex()))
                            .eq("tag_name", medicalIndicationDTO.getName())
                            .eq("type", QuestionTypeEnum.MedicalIndication.getKey());
                    //关联标签
                    QuestionInfo indication = questionFacade.getOne(questionInfoQueryWrapper);
                    if (indication != null) {
                        medicalIndicationDTO.setId(indication.getId());
                    }

                    //量表标签
                    if (medicalIndicationDTO.getType().equals(1)) {
                        questionInfoQueryWrapper = new QueryWrapper<>();
                        questionInfoQueryWrapper.le("age_begin", pushVO.getAge())
                                .ge("age_end", pushVO.getAge())
                                .and(i -> i.eq("sex_type", 3).or().eq("sex_type", pushVO.getSex()))
                                .eq("tag_name", medicalIndicationDTO.getScaleName())
                                .eq("type", QuestionTypeEnum.Scale.getKey());
                        QuestionInfo scaleInfo = questionFacade.getOne(questionInfoQueryWrapper);
                        ScaleDTO scaleDTO = new ScaleDTO();
                        if (scaleInfo != null) {
                            BeanUtil.copyProperties(scaleInfo, scaleDTO);
                        } else {
                            scaleDTO.setName(medicalIndicationDTO.getScaleName());
                            scaleDTO.setTagName(medicalIndicationDTO.getScaleName());
                        }
                        medicalIndicationDTO.setScale(scaleDTO);
                    }
                }
                pushDTO.setMedicalIndications(medicalIndicationDTOList);
            }
        }
        return pushDTO;
    }


    /**
     * 获取返回结果标签
     *
     * @param nameList
     * @param pushVO
     * @param type
     * @return
     */
    public List<QuestionDTO> getTagList(List<String> nameList, PushVO pushVO, Integer type) {
        List<QuestionDTO> questionDTOList = Lists.newLinkedList();
        QueryWrapper<QuestionInfo> questionInfoQueryWrapper = new QueryWrapper();
        questionInfoQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey()).
                in("tag_name", nameList).
                eq("type", type);
        List<QuestionInfo> questionInfoList = questionFacade.list(questionInfoQueryWrapper);
        Map<String, QuestionInfo> questionMap = EntityUtil.makeEntityMap(questionInfoList, "tagName");
        for (String name : nameList) {
            QuestionDTO questionDTO = new QuestionDTO();
            if (questionMap.get(name) != null) {
                QuestionVO questionVO = new QuestionVO();
                questionVO.setAge(pushVO.getAge());
                questionVO.setSexType(pushVO.getSex());
                questionVO.setId(questionMap.get(name).getId());
                questionDTO = questionFacade.getById(questionVO);
                if (questionDTO == null || questionDTO.getId() == null) {
                    questionDTO = new QuestionDTO();
                    questionDTO.setTagName(name);
                    questionDTO.setName(name);
                }
            } else {
                questionDTO.setTagName(name);
                questionDTO.setName(name);
            }
            questionDTOList.add(questionDTO);
        }
        return questionDTOList;
    }

    /**
     * 推理接口-快易检
     *
     * @param pushKYJVO
     * @return
     */
    public PushKYJDTO pushKYJ(PushKYJVO pushKYJVO) {
        PushVO pushVO = new PushVO();
        pushVO.setAge(pushKYJVO.getAge());
        pushVO.setSex(pushKYJVO.getSex());
        pushVO.setFeatureType(pushKYJVO.getFeatureType());

        //参数预处理
        /*String lis = "";
        List<String> lisResultList = Lists.newArrayList();
        for (LisKYJVO lisKYJVO : pushKYJVO.getLisResultList()) {
            lisKYJVO = resultCalculation(lisKYJVO);
            lisResultList.addAll(getLisInputs(lisKYJVO));
        }
        lis = String.join(",", lisResultList);
        pushVO.setLis(lis);*/

        //化验结果
        List<LisResultVO> lisResultVOList = BeanUtil.listCopyTo(pushKYJVO.getLisResultList(), LisResultVO.class);
        for (LisResultVO lisResultVO : lisResultVOList) {
            lisResultVO.setSource(LisSourceEnum.Outer.getKey());
        }
        pushVO.setLis(lisResultVOList);

        SearchData searchData = assembleData(pushVO);
        ResponseData responseData = pushAipt(searchData);
        PushKYJDTO pushKYJDTO = new PushKYJDTO();
        pushKYJDTO.setLabs(responseData.getLabs());
        pushKYJDTO.setDis(responseData.getDis());
        return pushKYJDTO;
    }


    /**
     * 快易检结果预处理
     *
     * @param lisKYJVO
     * @return
     */
    public LisKYJVO resultCalculation(LisKYJVO lisKYJVO) {
        if (lisKYJVO.getValue() != null) {
            if (lisKYJVO.getMaxValue() != null) {
                Integer comResult = lisKYJVO.getValue().compareTo(lisKYJVO.getMaxValue());
                if (comResult > 0) {
                    lisKYJVO.setMultipleState(QuantitativeTypeEnum.Up.getName());
                    if (!lisKYJVO.getMaxValue().equals(0.0)) {
                        lisKYJVO.setMultipleValue(lisKYJVO.getValue() / lisKYJVO.getMaxValue());
                    }
                }
            }
            if (lisKYJVO.getMinValue() != null) {
                Integer comResult = lisKYJVO.getValue().compareTo(lisKYJVO.getMinValue());
                if (comResult < 0) {
                    lisKYJVO.setMultipleState(QuantitativeTypeEnum.Down.getName());
                    if (!lisKYJVO.getMinValue().equals(0.0)) {
                        lisKYJVO.setMultipleValue(lisKYJVO.getValue() / lisKYJVO.getMinValue());
                    }
                }
            }
            if (StringUtil.isBlank(lisKYJVO.getMultipleState())) {
                lisKYJVO.setMultipleState(QuantitativeTypeEnum.Normal.getName());
            }
        }
        return lisKYJVO;
    }

    /**
     * 拼接化验项的文本结果
     *
     * @param lisKYJVO
     * @return
     */
    public List<String> getLisInputs(LisKYJVO lisKYJVO) {
        List<String> stringList = Lists.newArrayList();
        if (StringUtil.isBlank(lisKYJVO.getName())) {
            return stringList;
        } else if (lisKYJVO.getValue() != null) {
            //数值
            String lisText = lisKYJVO.getName() + "," + lisKYJVO.getValue() + " " + lisKYJVO.getUnits();
            stringList.add(lisText);
            //倍数
            if (StringUtil.isBlank(lisKYJVO.getMultipleState())) {
                lisText = lisKYJVO.getName() + "," + lisKYJVO.getMultipleState() + " " + lisKYJVO.getMultipleValue();
                stringList.add(lisText);
            }
        } else if (StringUtil.isBlank(lisKYJVO.getOtherValue())) {
            //阴性阳性和其他文字描述
            String lisText = lisKYJVO.getName() + "," + lisKYJVO.getOtherValue();
            stringList.add(lisText);
        }
        return stringList;
    }

    /**
     * 获取查体模板
     *
     * @param deptName
     * @param pushVO
     * @return
     */
    public List<QuestionDTO> getVitalModule(String deptName, PushVO pushVO) {
        List<QuestionDTO> vitalDTO = Lists.newLinkedList();
        QueryWrapper<DeptInfo> deptInfoQueryWrapper = new QueryWrapper<>();
        deptInfoQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey()).
                eq("name", deptName);
        DeptInfo deptInfo = deptInfoFacade.getOne(deptInfoQueryWrapper);
        if (deptInfo == null) {
            //科室信息不存在时推全科模板
            deptName = "全科";
            deptInfoQueryWrapper = new QueryWrapper<>();
            deptInfoQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey()).
                    eq("name", deptName);
            deptInfo = deptInfoFacade.getOne(deptInfoQueryWrapper);
            if (deptInfo == null) {
                throw new CommonException(CommonErrorCode.NOT_EXISTS, "全科信息未维护");
            }
        }

        QueryWrapper<DeptVital> deptVitalQueryWrapper = new QueryWrapper<>();
        deptVitalQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey()).
                eq("dept_id", deptInfo.getId()).
                orderByAsc("order_no");
        List<DeptVital> deptVitalList = deptVitalFacade.list(deptVitalQueryWrapper);
        //科室模板为空时，取全科模板
        if (deptVitalList.size() == 0) {
            deptVitalQueryWrapper = new QueryWrapper<>();
            String sql = "select id from icss_dept_info where name='全科' and is_deleted='N'";
            deptVitalQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey()).
                    inSql("dept_id", sql).
                    orderByAsc("order_no");
            deptVitalList = deptVitalFacade.list(deptVitalQueryWrapper);
        }
        //标签列表
        for (DeptVital deptVital : deptVitalList) {
            QuestionVO questionVO = new QuestionVO();
            questionVO.setAge(pushVO.getAge());
            questionVO.setSexType(pushVO.getSex());
            questionVO.setId(deptVital.getVitalId());
            QuestionDTO questionDTO = questionFacade.getById(questionVO);
            if (questionDTO != null && questionDTO.getId() != null) {
                vitalDTO.add(questionDTO);
            }
        }
        return vitalDTO;
    }

    /**
     * 电子病历评级-数据服务模式推理
     *
     * @param emrPushVO
     * @return
     */
    public PushEMRDTO pushEMR(EMRPushVO emrPushVO) {
        Boolean isDocking = tranServiceClient.isDocking().data;
        PushEMRDTO pushEMRDTO = new PushEMRDTO();
        HosCodeVO hosCodeVO = new HosCodeVO();
        hosCodeVO.setHosCode(emrPushVO.getHosCode());
        PushVO pushVO = new PushVO();
        BeanUtil.copyProperties(emrPushVO, pushVO);
        //化验项转公表内容,参数处理
        if (ListUtil.isNotEmpty(pushVO.getLis())) {
            Map<String, Map<String, String>> lisConfigMap = new LinkedHashMap<>();
            Map<String, String> lisConfigMap_emptyItemName = new LinkedHashMap<>();
            if (isDocking) {
                lisConfigMap = tranServiceClient.getLisConfigByHosCode_NotEmptyItemName(hosCodeVO).data;
                lisConfigMap_emptyItemName = tranServiceClient.getLisConfigByHosCode_EmptyItemName(hosCodeVO).data;
            } else {
                lisConfigMap = lisMappingFacade.getLisMapping_NotEmptyItemName();
                lisConfigMap_emptyItemName = lisMappingFacade.getLisConfig_EmptyItemName();
            }
            List<LisResultVO> lisResultVOList = pushVO.getLis();
            for (LisResultVO lisResultVO : lisResultVOList) {
                if (StringUtil.isNotBlank(lisResultVO.getName())) {
                    if (StringUtil.isNotBlank(lisResultVO.getDetailName())) {
                        lisResultVO.setUniqueName(lisConfigMap.get(lisResultVO.getName()).get(lisResultVO.getDetailName()));
                    } else {
                        lisResultVO.setUniqueName(lisConfigMap_emptyItemName.get(lisResultVO.getName()));
                    }
                }
            }
            pushVO.setLis(lisResultVOList);
        }

        SearchData searchData = assembleData(pushVO);
        ResponseData data = pushAipt(searchData);
        String featureType = pushVO.getFeatureType();
        String[] featureTypes = featureType.split(",|，");
        Set<String> featureTypeSet = new HashSet(Arrays.asList(featureTypes));

        if (featureTypeSet.contains(String.valueOf(QuestionTypeEnum.Lis.getKey()))) {
            List<FeatureRate> lisFeatures = data.getLabs();
            if (lisFeatures.size() > 0) {
                Map<String, List<String>> lisMappingByUniqueName = new LinkedHashMap<>();
                if (isDocking) {
                    lisMappingByUniqueName = tranServiceClient.getLisConfigByUniqueNameAndHosCode(hosCodeVO).data;
                } else {
                    lisMappingByUniqueName = lisMappingFacade.getLisMappingByUniqueName();
                }
                List<EMRQuestionDTO> lisList = Lists.newLinkedList();
                for (FeatureRate featureRate : lisFeatures) {
                    EMRQuestionDTO lis = new EMRQuestionDTO();
                    lis.setUniqueName(featureRate.getFeatureName());
                    if (ListUtil.isNotEmpty(lisMappingByUniqueName.get(featureRate.getFeatureName()))) {
                        lis.setName(lisMappingByUniqueName.get(featureRate.getFeatureName()).get(0));
                    }
                    lisList.add(lis);
                }
                pushEMRDTO.setLisList(lisList);
            }
        }
        if (featureTypeSet.contains(String.valueOf(QuestionTypeEnum.Pacs.getKey()))) {
            List<FeatureRate> pacsFeatures = data.getPacs();
            if (pacsFeatures.size() > 0) {
                Map<String, List<String>> pacsConfigMapByUniqueName = new LinkedHashMap<>();
                if (isDocking) {
                    pacsConfigMapByUniqueName = tranServiceClient.getPacsConfigByUniqueNameAndHosCode(hosCodeVO).data;
                }
                List<EMRQuestionDTO> pacsList = Lists.newLinkedList();
                for (FeatureRate featureRate : pacsFeatures) {
                    EMRQuestionDTO pacs = new EMRQuestionDTO();
                    pacs.setUniqueName(featureRate.getFeatureName());
                    if (isDocking) {
                        if (ListUtil.isNotEmpty(pacsConfigMapByUniqueName.get(featureRate.getFeatureName()))) {
                            pacs.setName(pacsConfigMapByUniqueName.get(featureRate.getFeatureName()).get(0));
                        }
                    } else {
                        pacs.setName(featureRate.getFeatureName());
                    }
                    pacsList.add(pacs);
                }
                pushEMRDTO.setPacsList(pacsList);
            }
        }
        if (featureTypeSet.contains(String.valueOf(QuestionTypeEnum.Disease.getKey()))) {
            List<FeatureRate> dis = data.getDis();
            List<String> disNameList = dis.stream().map(disease -> disease.getFeatureName()).collect(Collectors.toList());
            Map<String, List<EMRQuestionDTO>> retDisMap = new LinkedHashMap<>();
            Map<String, String> disMap = tranServiceClient.getDiseaseIcdByHosCode(hosCodeVO).data;

            //诊断分类
            for (FeatureRate featureRate : dis) {
                if (StringUtil.isBlank(featureRate.getDesc())) {
                    featureRate.setDesc("{\"可能诊断\":\"\"}");
                }
                Map<String, Object> descMap = FastJsonUtils.getJsonToMap(featureRate.getDesc());
                for (String disClass : descMap.keySet()) {
                    List<EMRQuestionDTO> retDisList = Lists.newLinkedList();
                    if (retDisMap.get(disClass) != null) {
                        retDisList = retDisMap.get(disClass);
                    }
                    EMRQuestionDTO retDis = new EMRQuestionDTO();
                    retDis.setUniqueName(featureRate.getFeatureName());
                    if (isDocking) {
                        //对接模式转icd10名称
                        retDis.setName(disMap.get(featureRate.getFeatureName()));
                    } else {
                        retDis.setName(featureRate.getFeatureName());
                    }
                    retDisList.add(retDis);
                    retDisMap.put(disClass, retDisList);
                }
            }
            //警惕
            searchData = new SearchData();
            searchData.setDiag(String.join(",", disNameList));
            Response<GdbResponse> graphRes = aiptServiceClient.highRisk(searchData);
            Map<String, String> graphResult = graphRes.getData().getResult();
            if (graphResult.size() > 0) {
                List<EMRQuestionDTO> highRiskDisList = Lists.newLinkedList();
                for (Map.Entry<String, String> entry : graphResult.entrySet()) {
                    if (entry.getValue().equals("1")) {
                        EMRQuestionDTO hrDis = new EMRQuestionDTO();
                        hrDis.setUniqueName(entry.getKey());
                        if (isDocking) {
                            hrDis.setName(disMap.get(entry.getKey()));
                        } else {
                            hrDis.setName(entry.getValue());
                        }
                        highRiskDisList.add(hrDis);
                    }
                }
                retDisMap.put("警惕", highRiskDisList);
            }
            pushEMRDTO.setDis(retDisMap);
        }
        return pushEMRDTO;
    }

    /**
     * 返回量表内容
     *
     * @param pushVO
     * @return
     */
    public String getScaleContent(PushVO pushVO) {
        if (StringUtil.isBlank(pushVO.getScaleName())) {
            if (pushVO.getScaleId() == null) {
                throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "请输入量表名称");
            } else {
                QuestionInfo scale = questionFacade.getById(pushVO.getScaleId());
                if (scale == null) {
                    throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "请输入正确的量表id");
                } else {
                    pushVO.setScaleName(scale.getTagName());
                }
            }
        }
        SearchData searchData = assembleData(pushVO);
        Map<String, Object> scale = scale(searchData);

        String scaleContent = "";
        if (scale != null && scale.size() > 0) {
            scaleContent = JSON.toJSONString(scale);
        }
        return scaleContent;
    }

    /**
     * 获取管理评估内容，titles不为空，返回指定内容，否则返回所有内容
     *
     * @param pushVO
     * @param titles
     * @return
     */
    public Map<String, JSONObject> getManagementEvaluationContent(PushVO pushVO, List<String> titles) {
        String[] diseases = pushVO.getDiag().split(",|，");

        if (ArrayUtil.isEmpty(diseases)) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "请输入诊断名称");
        }
        String disease = diseases[0];
        //判断是否为慢病
        if (!disTypeFacade.judgeByDisName(disease, DisTypeEnum.CHRONIC.getKey())) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "非慢病，不能获取管理评估内容");
        }

        SearchData searchData = assembleData(pushVO);
        ResponseData data = pushAipt(searchData);
        Map<String, JSONObject> managementEvaluationMap = data.getManagementEvaluation();
        if (managementEvaluationMap == null) {
            managementEvaluationMap = new LinkedHashMap<>();
            managementEvaluationMap.put("疗效评估", JSONObject.parseObject("{\"list\":[{\"className\":\"血压控制情况\",\"controltype\":1,\"details\":[{\"name\":\"正常\",\"isSelected\":1},{\"name\":\"1级\",\"isSelected\":0},{\"name\":\"2级\",\"isSelected\":0},{\"name\":\"3级\",\"isSelected\":0}]},{\"className\":\"BMI控制情况\",\"controltype\":2,\"details\":[{\"name\":\"体重过低\",\"isSelected\":1},{\"name\":\"体重达标\",\"isSelected\":0},{\"name\":\"肥胖前期\",\"isSelected\":0},{\"name\":\"肥胖\",\"isSelected\":0}]},{\"className\":\"血脂控制情况\",\"controltype\":2,\"details\":[{\"name\":\"高血脂\",\"isSelected\":1}]}]}"));
            managementEvaluationMap.put("糖尿病分级管理", JSONObject.parseObject("{\"className\":\"血压控制情况\",\"controltype\":1,\"details\":[{\"name\":\"常规管理\",\"isSelected\":1},{\"name\":\"强化管理\",\"isSelected\":0}]}"));
        }
        Map<String, JSONObject> retMap = new LinkedHashMap<>();
        if (ListUtil.isEmpty(titles)) {
            retMap = managementEvaluationMap;
        } else {
            for (String title : titles) {
                if (managementEvaluationMap.get(title) != null) {
                    retMap.put(title, managementEvaluationMap.get(title));
                }
            }
        }
        return retMap;
    }
}