package com.diagbot.facade;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.client.bean.AdverseReaction;
import com.diagbot.client.bean.Medicition;
import com.diagbot.client.bean.MedicitionClass;
import com.diagbot.dto.IntroduceDetailDTO;
import com.diagbot.entity.InquiryDrug;
import com.diagbot.entity.InquiryInfo;
import com.diagbot.entity.IntroduceDetail;
import com.diagbot.entity.IntroduceMap;
import com.diagbot.entity.QuestionInfo;
import com.diagbot.enums.DisTypeEnum;
import com.diagbot.enums.IntroducePositionEnum;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.QuestionTypeEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.FastJsonUtils;
import com.diagbot.util.ListUtil;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2018/12/17 13:41
 */
@Component
public class TreatmentFacade {
    @Autowired
    QuestionFacade questionFacade;
    @Autowired
    IntroduceDetailFacade introduceDetailFacade;
    @Autowired
    IntroduceMapFacade introduceMapFacade;
    @Autowired
    DisTypeFacade disTypeFacade;
    @Autowired
    InquiryInfoFacade inquiryInfoFacade;
    @Autowired
    InquiryDrugFacade inquiryDrugFacade;

    /**
     * 获取治疗方案
     *
     * @param
     * @return
     */
    public Map<String, Object> getTreatment(Map<String, JSONObject> treatmentMap, Long diseaseId, Long patientId) {
        Map<String, Object> retMap = new LinkedHashMap<>();
        QuestionInfo disease = questionFacade.getById(diseaseId);
        if (disease == null) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "诊断不存在");
        }
        if (treatmentMap.size() == 0) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "未找到治疗方案");
        }
        if (!treatmentMap.containsKey(disease.getTagName())) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "诊断名称和id不匹配");
        }

        //一般治疗
        List<IntroduceDetail> commonTreatmentDetailList = getIntroduceDetailList(diseaseId, IntroducePositionEnum.CommonTreatment.getKey());
        IntroduceDetailDTO commonDetailDTO = new IntroduceDetailDTO();
        if (ListUtil.isNotEmpty(commonTreatmentDetailList)) {
            BeanUtil.copyProperties(commonTreatmentDetailList.get(0), commonDetailDTO);
        }
        retMap.put("commonTreatment", commonDetailDTO);

        //手术治疗
        List<IntroduceDetail> surgeryTreatmentDetailList = getIntroduceDetailList(diseaseId, IntroducePositionEnum.SurgeryTreatment.getKey());
        IntroduceDetailDTO surgeryDetailDTO = new IntroduceDetailDTO();
        if (ListUtil.isNotEmpty(surgeryTreatmentDetailList)) {
            BeanUtil.copyProperties(surgeryTreatmentDetailList.get(0), surgeryDetailDTO);
        }
        retMap.put("surgeryTreatment", surgeryDetailDTO);


        //获取知识图谱治疗方案
        JSONObject treatmentJson = treatmentMap.get(disease.getTagName());
        if (treatmentJson == null || treatmentJson.isEmpty()) {
            return retMap;
        }

        List<MedicitionClass> drugsList = FastJsonUtils.getJsonToListByKey(treatmentJson.toString(), "treatment", MedicitionClass.class);

        List<String> classNameList = drugsList.stream().map(drugs -> drugs.getBigdrugsName()).collect(Collectors.toList());
        List<QuestionInfo> drugClassList = getTopQuestionList(classNameList, QuestionTypeEnum.DrugClass.getKey());
        Map<String, QuestionInfo> drugClassMap = EntityUtil.makeEntityMap(drugClassList, "tagName");
        List<Long> drugClassIds = drugClassList.stream().map(drugClass -> drugClass.getId()).collect(Collectors.toList());
        List<IntroduceMap> introduceMapList = getIntroduceMapList(drugClassIds, QuestionTypeEnum.DrugClass.getKey());
        Map<Long, IntroduceMap> introduceMapMap = EntityUtil.makeEntityMap(introduceMapList, "questionId");

        for (MedicitionClass medicitionClass : drugsList) {
            QuestionInfo drugClass = drugClassMap.get(medicitionClass.getBigdrugsName());
            if (drugClass != null) {
                medicitionClass.setId(drugClass.getId());
                if (introduceMapMap.get(drugClass.getId()) != null) {
                    medicitionClass.setShowInfo("1");
                }
            }
            LinkedList<Medicition> medicitionList = medicitionClass.getMedicitionsList();
            List<String> drugNameList = medicitionList.stream().map(medicition -> medicition.getMedicitionName()).collect(Collectors.toList());
            List<QuestionInfo> drugList = getTopQuestionList(drugNameList, QuestionTypeEnum.Drug.getKey());
            Map<String, QuestionInfo> drugMap = EntityUtil.makeEntityMap(drugList, "tagName");
            List<Long> drugIds = drugList.stream().map(drug -> drug.getId()).collect(Collectors.toList());
            List<IntroduceMap> drugIntroduceMapList = getIntroduceMapList(drugIds, QuestionTypeEnum.Drug.getKey());
            Map<Long, IntroduceMap> drugIntroduceMapMap = EntityUtil.makeEntityMap(drugIntroduceMapList, "questionId");
            for (Medicition medicition : medicitionList) {
                QuestionInfo drug = drugMap.get(medicition.getMedicitionName());
                if (drug != null) {
                    medicition.setId(drug.getId());
                    if (drugIntroduceMapMap.get(drug.getId()) != null) {
                        medicition.setShowInfo("1");
                    }
                }
            }
            medicitionClass.setMedicitionsList(medicitionList);
        }

        retMap.put("treatment", drugsList);

        //判断是否慢病，慢病增加回访时间，上次用药，不良反应
        Boolean isChronic = disTypeFacade.judgeByDisId(diseaseId, DisTypeEnum.CHRONIC.getKey());
        if (isChronic) {
            //慢病，增加回访时间，暂时前端写死， 以后由只是图谱返回
            retMap.put("followUp", null);

            //上次用药
            if (patientId == null) {
                retMap.put("drugHistory", null);
            } else {
                retMap.put("drugHistory", getDrugHistory(patientId));
            }

            //不良反应
            List<AdverseReaction> adverseReactionList = FastJsonUtils.getJsonToListByKey(treatmentJson.toString(), "adverseEvent", AdverseReaction.class);
            retMap.put("adverseReactions", adverseReactionList);
        }
        return retMap;
    }


    /**
     * 根据展示位置获取提示信息明细列表
     *
     * @param diseaseId
     * @param position
     * @return
     */
    public List<IntroduceDetail> getIntroduceDetailList(Long diseaseId, Integer position) {
        QueryWrapper<IntroduceDetail> introduceDetailQueryWrapper = new QueryWrapper<>();
        String sql = "select introduce_id from icss_introduce_map where question_id=" + diseaseId + " and type=" + QuestionTypeEnum.Disease.getKey();
        introduceDetailQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .apply("find_in_set({0},position)", position)
                .inSql("introduce_id", sql)
                .orderByAsc("introduce_id", "order_no");
        List<IntroduceDetail> introduceDetailList = introduceDetailFacade.list(introduceDetailQueryWrapper);
        return introduceDetailList;
    }

    /**
     * 获取顶级标签
     *
     * @param tagNameList
     * @param type
     * @return
     */
    public List<QuestionInfo> getTopQuestionList(List<String> tagNameList, Integer type) {
        QueryWrapper<QuestionInfo> questionInfoQueryWrapper = new QueryWrapper<>();
        questionInfoQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .in("tag_name", tagNameList)
                .eq("sub_type", 0)
                .eq("type", type);
        List<QuestionInfo> questionInfoList = questionFacade.list(questionInfoQueryWrapper);
        return questionInfoList;
    }

    /**
     * 获取标签提示信息映射
     *
     * @param questionIdList
     * @param type
     * @return
     */
    public List<IntroduceMap> getIntroduceMapList(List<Long> questionIdList, Integer type) {
        QueryWrapper<IntroduceMap> introduceMapQueryWrapper = new QueryWrapper<>();
        introduceMapQueryWrapper.in("question_id", questionIdList)
                .eq("type", type)
                .eq("is_deleted", IsDeleteEnum.N.getKey());
        List<IntroduceMap> introduceMapList = introduceMapFacade.list(introduceMapQueryWrapper);
        return introduceMapList;
    }

    /**
     * 根据病人id获取历史用药内容
     *
     * @param patientId
     * @return
     */
    public Map<String, List<Medicition>> getDrugHistory(Long patientId) {
        Map<String, List<Medicition>> drugHistoryMap = new LinkedHashMap<>();
        QueryWrapper<InquiryInfo> inquiryInfoQueryWrapper = new QueryWrapper<>();
        inquiryInfoQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey()).
                eq("patient_id", patientId).
                orderByDesc("gmt_modified");
        List<InquiryInfo> inquiryInfoList = inquiryInfoFacade.list(inquiryInfoQueryWrapper);
        Boolean chronicHistroy = false;
        Boolean not_chronicHistory = false;
        for (InquiryInfo inquiryInfo : inquiryInfoList) {
            String[] diseases = inquiryInfo.getDiagnose().split(",|，");
            if (diseases.length > 0) {
                QueryWrapper<InquiryDrug> inquiryDrugQueryWrapper = new QueryWrapper<>();
                inquiryDrugQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey()).eq("inquiry_id", inquiryInfo.getId());
                List<InquiryDrug> inquiryDrugList = inquiryDrugFacade.list(inquiryDrugQueryWrapper);
                List<Medicition> medicitionList = Lists.newLinkedList();
                for (InquiryDrug inquiryDrug : inquiryDrugList) {
                    Medicition medicition = new Medicition();
                    medicition.setId(inquiryDrug.getQuestionId());
                    medicition.setMedicitionName(inquiryDrug.getTagName());
                    medicition.setIsShow("1");
                    Boolean hasIntroduce = introduceMapFacade.hasIntroduceByQuestionName(inquiryDrug.getTagName(), QuestionTypeEnum.Drug.getKey());
                    if (hasIntroduce) {
                        medicition.setShowInfo("1");
                    } else {
                        medicition.setShowInfo("0");
                    }
                    medicitionList.add(medicition);
                }
                Boolean isChronic = disTypeFacade.judgeByDisName(diseases[0], DisTypeEnum.CHRONIC.getKey());
                if (isChronic) {
                    if (!chronicHistroy)
                        drugHistoryMap.put("慢病用药内容", medicitionList);
                    chronicHistroy = true;
                } else {
                    if (!not_chronicHistory) {
                        drugHistoryMap.put("普通病用药内容", medicitionList);
                        not_chronicHistory = true;
                    }
                }
            }
        }
        return drugHistoryMap;
    }
}