package com.diagbot.web;

import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.ItemIndexDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.IndexDataFacade;
import com.diagbot.vo.IndexDataFindVO;
import com.diagbot.vo.IndexDataSaveVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 指标数据表 前端控制器
 * </p>
 *
 * @author wangfeng
 * @since 2019-03-11
 */

@RestController
@RequestMapping("/indexData")
@Api(value = "指标数据API[by:wangfeng]", tags = { "WF-2期-指标数据API" })
@SuppressWarnings("unchecked")
public class IndexDataController {

    @Autowired
    IndexDataFacade indexDataFacade;

    @ApiOperation(value = "2期-根据疾病Id获取指标配置[by:wangfeng]", notes = "diseaseId：疾病Id 必填<br> patientId:患者id ,必填<br> startTime:开始时间 ,必填<br> endTime:结束时间 ,必填<br> ")
    @PostMapping("/getIndexDatas")
    @SysLogger("getIndexDatas")
    @Transactional
    public RespDTO<List<ItemIndexDTO>> getIndexDatas(@Valid @RequestBody IndexDataFindVO indexDataFindVO) {

        List<ItemIndexDTO> data = indexDataFacade.getIndexDataAll(indexDataFindVO);

        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "2期-保存指标数据[by:wangfeng]", notes = "patientId : 患者id 必填<br> inquiryCode ：就诊号  必填<br> doctorId：医生id必填<br> IndexData：{指标公表名 :indexUnique;指标值indexValue;指标单位 indexUnit;是否异常 isAbnormal;化验时间creatTime;}")
    @PostMapping("/saveIndexDatas")
    @SysLogger("saveIndexDatas")
    @Transactional
    public RespDTO<Boolean> saveIndexDatas(@Valid @RequestBody IndexDataSaveVO indexDataSaveVO) {

        boolean res = indexDataFacade.saveIndexDatas(indexDataSaveVO);

        return RespDTO.onSuc(res);
    }

}
