package com.diagbot.web;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.GetDisScaleAllInfoDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.DisScaleFacade;
import com.diagbot.vo.AddDisScaleInfoVO;
import com.diagbot.vo.DelDisScaleInfoVO;
import com.diagbot.vo.GetDisScaleAllInfoVO;
import com.diagbot.vo.GetDisScaleByDisIdVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 诊断量表映射表 前端控制器
 * </p>
 *
 * @author wangyu
 * @since 2019-04-01
 */
@RestController
@RequestMapping("/disScale")
@Api(value = "诊断关联量表相关API", tags = { "2期-诊断关联量表相关API" })
@SuppressWarnings("unchecked")
public class DisScaleController {

    @Autowired
    private DisScaleFacade disScaleFacade;

    @ApiOperation(value = "2期-诊断量表关联——添加和修改[by:wangyu]",
            notes = "disId: 诊断id,必填<br>" +
                    "scaleId: 量表id，必填<br>")
    @PostMapping("/addDisScaleInfo")
    @SysLogger("addDisScaleInfo")
    @Transactional
    public RespDTO<Boolean> addDisScaleInfo(@RequestBody @Valid AddDisScaleInfoVO addDisScaleInfoVO){
        Boolean data = disScaleFacade.addDisScaleInfo(addDisScaleInfoVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "2期-诊断量表关联——分页获取关联信息[by:wangyu]",
            notes = "disName: 诊断名称<br>" +
                    "scaleName: 量表名称<br>")
    @PostMapping("/getDisScaleAllInfo")
    @SysLogger("getDisScaleAllInfo")
    public RespDTO<IPage<GetDisScaleAllInfoDTO>> getDisScaleAllInfo(@RequestBody GetDisScaleAllInfoVO getDisScaleAllInfoVO) {
        IPage<GetDisScaleAllInfoDTO> data = disScaleFacade.getDisScaleAllInfo(getDisScaleAllInfoVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "2期-诊断量表关联——根据诊断id获取诊断量表关联信息(修改回读用)[by:wangyu]",
            notes = "disId: 诊断id,必填<br>")
    @PostMapping("/getDisScaleByDisId")
    @SysLogger("getDisScaleByDisId")
    public RespDTO<List<GetDisScaleAllInfoDTO>> getDisScaleByDisId(@Valid @RequestBody GetDisScaleByDisIdVO getDisScaleByDisIdVO) {
        List<GetDisScaleAllInfoDTO> data = disScaleFacade.getDisScaleByDisId(getDisScaleByDisIdVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "2期-诊断量表关联——删除[by:wangyu]",
            notes = "id: 分页查询主页面的id <br>" +
                    "<br>" +
                    "<br>")
    @PostMapping("/delDisScaleInfo")
    @SysLogger("delDisScaleInfo")
    @Transactional
    public RespDTO<Boolean> delDisScaleInfo(@RequestBody @Valid DelDisScaleInfoVO delDisScaleInfoVO){
        Boolean data = disScaleFacade.delDisScaleInfo(delDisScaleInfoVO);
        return RespDTO.onSuc(data);
    }
}
