package com.diagbot.web;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.EvaluationModuleAllDTO;
import com.diagbot.dto.EvaluationModuleMappingDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.EvaluationModuleMappingFacade;
import com.diagbot.vo.EvaluationModuleDiseaseIdVO;
import com.diagbot.vo.EvaluationModuleDiseaseNameVO;
import com.diagbot.vo.EvaluationModuleListVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * <p>
 * 管理与评估模板表 前端控制器
 * </p>
 *
 * @author wangfeng
 * @since 2019-03-25
 */
@RestController
@RequestMapping("/evaluationModuleMapping")
@Api(value = "慢病评估维护API[by:wangfeng]", tags = { "WF-2期-慢病评估维护API" })
@SuppressWarnings("unchecked")
public class EvaluationModuleMappingController {

    @Autowired
    EvaluationModuleMappingFacade evaluationModuleMappingFacade;


    @ApiOperation(value = "2期-分页模糊查询标签组维护接口[by:wangfeng]", notes = "diseaseName : 疾病名")
    @PostMapping("/queryEvaluationModulePages")
    @SysLogger("queryEvaluationModulePages")
    public RespDTO<IPage<EvaluationModuleMappingDTO>> queryEvaluationModulePages(@RequestBody EvaluationModuleDiseaseNameVO evaluationModuleDiseaseNameVO) {

        IPage<EvaluationModuleMappingDTO> data
                = evaluationModuleMappingFacade.queryEvaluationModulePage(evaluationModuleDiseaseNameVO);

        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "2期-获取慢病标签组维护[by:wangfeng]", notes = "疾病ID ： diseaseId")
    @PostMapping("/getEvaluationModuleLists")
    @SysLogger("getEvaluationModuleLists")
    public RespDTO<List<EvaluationModuleAllDTO>> getEvaluationModuleLists(@RequestBody EvaluationModuleDiseaseIdVO evaluationModuleDiseaseIdVO) {

        List<EvaluationModuleAllDTO> data = evaluationModuleMappingFacade.getEvaluationModuleList(evaluationModuleDiseaseIdVO);

        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "2期-慢病评估维护——修改管理评估[by:wangfeng]", notes = "diseaseId ：疾病id </br>"
            + "evaluationModuleData:{ 区域名 regionName;"
            + "区域类型（0默认自定义，1大数据，2内部数据）regionType; "
            + "区域顺序 ：orderNo; "
            + "疾病id： diseaseId;"
            + "备注：remark;} ")
    @PostMapping("/updateEvaluationModuleLists")
    @SysLogger("updateEvaluationModuleLists")
    @Transactional
    public RespDTO<Boolean> updateEvaluationModuleLists(@RequestBody EvaluationModuleListVO evaluationModuleListVO) {

        boolean res = evaluationModuleMappingFacade.updateEvaluationModuleList(evaluationModuleListVO);

        return RespDTO.onSuc(res);
    }

    @ApiOperation(value = "2期-慢病评估维护——添加管理评估[by:wangfeng]", notes = "diseaseId ：疾病id </br>"
            + "evaluationModuleData:{ 区域名 regionName;"
            + "区域类型（0默认自定义，1大数据，2内部数据）regionType; "
            + "区域顺序 ：orderNo; "
            + "疾病id： diseaseId;"
            + "备注：remark;} ")
    @PostMapping("/saveEvaluationModuleLists")
    @SysLogger("saveEvaluationModuleLists")
    @Transactional
    public RespDTO<Boolean> saveEvaluationModuleLists(@RequestBody EvaluationModuleListVO evaluationModuleListVO) {

        boolean res = evaluationModuleMappingFacade.saveEvaluationModuleList(evaluationModuleListVO);

        return RespDTO.onSuc(res);
    }

    @ApiOperation(value = "2期-删除慢病标签组维护[by:wangfeng]", notes = "diseaseId ：疾病id")
    @PostMapping("/cancelEvaluationModuleAlls")
    @SysLogger("cancelEvaluationModuleAlls")
    @Transactional
    public RespDTO<Boolean> cancelEvaluationModuleAlls(@RequestBody EvaluationModuleDiseaseIdVO evaluationModuleDiseaseIdVO) {

        boolean res = evaluationModuleMappingFacade.cancelEvaluationModuleAll(evaluationModuleDiseaseIdVO);

        return RespDTO.onSuc(res);
    }
}
