package com.diagbot.web;


import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.IndexConfigAllDTO;
import com.diagbot.dto.IndexConfigDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.IndexConfigFacade;
import com.diagbot.vo.IndexConfigDiseaseIdVO;
import com.diagbot.vo.IndexConfigDiseaseNameVO;
import com.diagbot.vo.IndexConfigListVO;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * <p>
 * 指标配置 前端控制器
 *
 * @author wangfeng
 * @since 2019-03-25
 */
@RestController
@RequestMapping("/indexConfig")
@Api(value = "慢病指标值关联维护API[by:wangfeng]", tags = { "WF-2期-慢病指标值关联维护API" })
@SuppressWarnings("unchecked") 
public class IndexConfigController {
	
	@Autowired
	IndexConfigFacade indexConfigFacade;
	
	@ApiOperation(value = "分页模糊查询慢病指标值关联维护[by:wangfeng]", notes = "")
    @PostMapping("/queryIndexConfigPages")
    @SysLogger("queryIndexConfigPages")
    @Transactional
    public RespDTO<IPage<IndexConfigDTO>> queryIndexConfigPages(@RequestBody IndexConfigDiseaseNameVO indexConfigDiseaseNameVO) {

    	IPage<IndexConfigDTO> data = indexConfigFacade.queryIndexConfigPage(indexConfigDiseaseNameVO);

        return RespDTO.onSuc(data);
    }

	@ApiOperation(value = "获取慢病指标值关联维护[by:wangfeng]", notes = "")
    @PostMapping("/getIndexConfigLists")
    @SysLogger("getIndexConfigLists")
    @Transactional
    public RespDTO<List<IndexConfigAllDTO>> getIndexConfigLists(@RequestBody IndexConfigDiseaseIdVO indexConfigDiseaseIdVO) {

		List<IndexConfigAllDTO> data = indexConfigFacade.getIndexConfigLists(indexConfigDiseaseIdVO);

        return RespDTO.onSuc(data);
    }
	
	@ApiOperation(value = "慢病指标值关联维护——修改关联[by:wangfeng]", notes = "")
    @PostMapping("/updateIndexConfigList")
    @SysLogger("updateIndexConfigList")
    @Transactional
    public RespDTO<Boolean> updateIndexConfigList(@RequestBody IndexConfigListVO indexConfigListVO) {

		boolean res = indexConfigFacade.updateIndexConfigList(indexConfigListVO);

        return RespDTO.onSuc(res);
    }
	@ApiOperation(value = "慢病指标值关联维护——添加关联[by:wangfeng]", notes = "")
    @PostMapping("/saveIndexConfigLists")
    @SysLogger("saveIndexConfigLists")
    @Transactional
    public RespDTO<Boolean> saveIndexConfigLists(@RequestBody IndexConfigListVO indexConfigListVO) {

		boolean res = indexConfigFacade.saveIndexConfigList(indexConfigListVO);

        return RespDTO.onSuc(res);
    }
	
	@ApiOperation(value = "删除慢病指标值关联维护[by:wangfeng]", notes = "")
    @PostMapping("/cancelIndexConfigAlls")
    @SysLogger("cancelIndexConfigAlls")
    @Transactional
    public RespDTO<Boolean> cancelIndexConfigAlls(@RequestBody IndexConfigDiseaseIdVO indexConfigDiseaseIdVO) {

		boolean res = indexConfigFacade.cancelIndexConfigAlls(indexConfigDiseaseIdVO);

        return RespDTO.onSuc(res);
    }
}
