package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.dto.ConceptDTO;
import com.diagbot.dto.LibraryDetailDTO;
import com.diagbot.entity.Concept;
import com.diagbot.entity.LibraryDetail;
import com.diagbot.entity.LibraryInfo;
import com.diagbot.service.impl.LibraryDetailServiceImpl;
import com.diagbot.vo.AmendTermVo;
import com.diagbot.vo.ConceptVo;
import com.diagbot.vo.KnowledgeVo;
import com.diagbot.vo.TermVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.RequestBody;

import java.time.LocalDateTime;
import java.util.*;

/**
 * @Description: 术语基本信息查询业务层
 * @author: Weixuan Huang
 * @time: 2019/1/16 14:17
 */
@Component
public class LibraryDetailFacade extends LibraryDetailServiceImpl {

    @Autowired
    private ConceptFacade conceptFacade;

    @Autowired
    private LibraryInfoFacade libraryInfoFacade;


    /**
     * 获取所有术语静态知识列表
     *
     * @param conceptVo
     * @return IPage<LibraryDetailDTO>
     */
    public IPage<LibraryDetailDTO> getAllLibraryDetail(ConceptVo conceptVo) {

        return getAllKnowledge(conceptVo);
    }




    /**
     * 获取医学术语静态信息
     *
     * @param knowledgeVo
     * @return ConceptDTO
     */
    public ConceptDTO getConceptKnowledge(@RequestBody KnowledgeVo knowledgeVo) {

        LibraryInfo libraryInfo;
        List<LibraryDetail> libraryDetails = new ArrayList<>();
        TermVo termVo = new TermVo();
        termVo.setTerm(knowledgeVo.getTerm());
        termVo.setType(knowledgeVo.getType());

        ConceptDTO conceptDTO = conceptFacade.getConcept(termVo);

        if (conceptDTO != null) {
            // 获取与医学术语的基本信息
            QueryWrapper<LibraryInfo> query = new QueryWrapper<>();
            query.eq("concept_id", conceptDTO.getId());
            query.eq("is_deleted", "N");
            query.eq("is_concept", 1);
            libraryInfo = libraryInfoFacade.getOne(query);

            conceptDTO.setName(libraryInfo.getName());

            // 获取与术语相关的静态知识
            QueryWrapper<LibraryDetail> querydetail = new QueryWrapper<>();
            querydetail.eq("concept_id", conceptDTO.getId());
            querydetail.eq("is_deleted", "N");

            if (knowledgeVo.getSource() != null && knowledgeVo.getSource().trim().length() > 0) {
                querydetail.eq("source", knowledgeVo.getSource());
            }

            if (knowledgeVo.getTitle() != null && knowledgeVo.getTitle().trim().length() > 0) {
                querydetail.eq("title", knowledgeVo.getTitle());
            }

            libraryDetails = list(querydetail);

            if (libraryDetails != null) {
                conceptDTO.setInformation(libraryDetails);
            }

        }

        return conceptDTO;
    }


    /**
     * 更新或添加医学术语静态信息
     *
     * @param amendTermVo
     * @return ConceptDTO
     */
    public ConceptDTO upsertConceptKnowledge(AmendTermVo amendTermVo) {

        ConceptDTO conceptDTO = new ConceptDTO();

        LibraryDetail libraryDetail;
        List<LibraryDetail> libraryDetails;

        conceptDTO.setName(amendTermVo.getNewterm());
        QueryWrapper<LibraryInfo> query = new QueryWrapper<>();
        query.eq("name", amendTermVo.getNewterm());
        query.eq("type", amendTermVo.getNewtype());
        LibraryInfo libraryInfo = libraryInfoFacade.getOne(query);

        if (amendTermVo.getInformation() != null && libraryInfo != null) {

            conceptDTO.setLibId(libraryInfo.getId().intValue());
            conceptDTO.setId(libraryInfo.getConceptId());
            conceptDTO.setMedicalInfo(new ArrayList<>());

            for (int i=0; i<amendTermVo.getInformation().size(); i++) {
                libraryDetail = amendTermVo.getInformation().get(i);

                if (libraryDetail.getTitle() != null && libraryDetail.getSource() != null) {
                    QueryWrapper<LibraryDetail> detquery = new QueryWrapper<>();
                    detquery.eq("concept_id", libraryInfo.getConceptId());
                    detquery.eq("title", libraryDetail.getTitle());
                    detquery.eq("source", libraryDetail.getSource());
                    LibraryDetail exist = getOne(detquery);

                    if (exist != null) {
                        libraryDetail.setId(exist.getId());
                    }

                    libraryDetail.setConceptId(libraryInfo.getConceptId().longValue());
                    libraryDetail.setGmtModified(LocalDateTime.now());
                    libraryDetail.setIsDeleted("N");

                    this.saveOrUpdate(libraryDetail);

                    if (conceptDTO.getInformation() == null) {
                        conceptDTO.setInformation(new ArrayList<LibraryDetail>());
                    }

                    conceptDTO.getInformation().add(libraryDetail);
                }
            }
        }
        else {
            conceptDTO.setMessage(new ArrayList<>());
            conceptDTO.getMessage().add("术语:\"" + amendTermVo.getNewterm() +
                    "\" 类型:\"" + amendTermVo.getNewtype() + "\" 不存在! ");
            conceptDTO.setInformation(null);
        }

        return conceptDTO;
    }

    /**
     * 删除术语医学信息
     *
     * @param knowledgeVo
     * @return ConceptDTO
     */
    public ConceptDTO removeConceptKnowledge(KnowledgeVo knowledgeVo) {

        ConceptDTO conceptDTO = new ConceptDTO();
        List<String> msg = new ArrayList<>();
        String error = "";
        LibraryDetail libraryDetail;

        /*
        QueryWrapper<LibraryInfo> query = new QueryWrapper<>();
        query.eq("name", knowledgeVo.getTerm());
        query.eq("type", knowledgeVo.getType());
        LibraryInfo libraryInfo = libraryInfoFacade.getOne(query);

        if (libraryInfo != null) {
            QueryWrapper<LibraryDetail> detquery = new QueryWrapper<>();
            detquery.eq("concept_id", libraryInfo.getConceptId());
            if (knowledgeVo.getSource() != null) {
                detquery.eq("source", knowledgeVo.getSource());
            }
            detquery.eq("title", knowledgeVo.getTitle());
            libraryDetail = getOne(detquery);

            if (libraryDetail != null) {
                libraryDetail.setIsDeleted("Y");
                saveOrUpdate(libraryDetail);
//                isdeleted = true;
            }
            else {
                error = "术语 \"" + knowledgeVo.getTerm() + "(" + knowledgeVo.getType() + ")\" " +
                        "的 " + knowledgeVo.getSource() + "-" + knowledgeVo.getTitle() + " 不存在!";
            }
        }
        else {
            error = "术语 \"" + knowledgeVo.getTerm() + "(" + knowledgeVo.getType() + ")\" 不存在!";
        }
        */
        QueryWrapper<LibraryDetail> query = new QueryWrapper<>();
        query.eq("id", knowledgeVo.getId());
        libraryDetail = getOne(query);

        if (libraryDetail != null) {
            remove(query);
        }
        else {
            error = "术语 \"" + knowledgeVo.getTerm() + "(" + knowledgeVo.getType() + ")\" 不存在!";
        }


        msg.add(error);
        conceptDTO.setMessage(msg);
        return conceptDTO;
    }

    /**
     * 清空医学术语知识表
     *
     */
    public void deleteDetail() {
        truncateDetail();
    }
}
